

class EventBase(object):
    event_name: str

    def run(self, **data):
        pass


class WalletCreateEvent(EventBase):
    event_name = "wallet_created"
    event_type = "Wallet"

    def run(self, **data):
        pass



from typing import TypeVar, Optional

from pydantic.generics import GenericModel, Generic
from .utils import fingerprintHash, fingerprintSignature
from typing import Optional
from pydantic import Extra, BaseModel


T = TypeVar('T')


class ResponseData(GenericModel, Generic[T]):
    status_code: Optional[int]
    url: Optional[str]
    data: Optional[T]
    message: Optional[str]
    error_description: Optional[str]
    error: Optional[str]
    error_code: Optional[str]


class JWTData(BaseModel):
    refresh_token: str
    token_type: str
    access_token: str
    expires_in: int


class HeadersSoldoBase(BaseModel):
    """
    headers for Standard Authentication
    http://apidoc-demo.soldo.com/v2/zgxiaxtcyapyoijojoef.html#standard-authentication
    """
    Authorization: Optional[str]

    def __init__(self, *args, **kwargs):
        from vc.client.soldo.client import Soldo
        super().__init__(*args, **kwargs)
        self.Authorization = f"Bearer {Soldo.settings.ACCESS_TOKEN}"

    class Config:
        extra = Extra.allow


class HeadersSoldo(HeadersSoldoBase):
    """
    headers for Advanced Authentication
    http://apidoc-demo.soldo.com/v2/zgxiaxtcyapyoijojoef.html#advanced-authentication
    """
    Content_Type: str = "application/json"
    fingerprintH: Optional[str]
    fingerprintS: Optional[str]

    def __init__(self, data, fields=None, **kwargs):
        from vc.client.soldo.client import Soldo
        super().__init__(**kwargs)
        if not fields:
            fields = data.keys()
        fingerprint = ""
        for field in fields:
            if data.get(field):
                fingerprint += str(data.get(field))
                print(fingerprint)
        fingerprint += Soldo.settings.TOKEN
        print(fingerprint)
        self.fingerprintH = fingerprintHash(fingerprint)
        self.fingerprintS = fingerprintSignature(self.fingerprintH, Soldo.settings.PATH_RSA_PRIVATE)

    class Config:
        fields = {
            "fingerprintS": "X-Soldo-Fingerprint-Signature",
            "fingerprintH": "X-Soldo-Fingerprint",
        }
        extra = Extra.allow



from contextlib import contextmanager
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker, scoped_session


class BaseNetworkClient(object):
    _user = None

    def __init__(self, uri: str, pool_timeout=10,
                 pool_size=5, max_overflow=100,
                 connect_timeout=10,
                 session_name="virtual_card", celery_broker=None, celery_backend=None, **kwargs):
        pass
        # self.engine = create_engine(uri, pool_pre_ping=True,
        #                             pool_timeout=pool_timeout,
        #                             pool_size=pool_size, max_overflow=max_overflow,
        #                             connect_args={'connect_timeout': connect_timeout,
        #                                           "application_name": session_name})
        # self.SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=self.engine)

    @contextmanager
    def get_session(self):
        session = None
        try:
            session = self.SessionLocal()
            yield session
        except Exception as exc:
            try:
                session.rollback()
            except Exception:
                pass
            raise exc
        finally:
            if session:
                session.close()

    def oauth_authorize(self):
        raise NotImplementedError

    def whoami(self):
        raise NotImplementedError

    def user_update(self, id:int, **kwargs):
        raise NotImplementedError

    def create_user(self,email: str, name: str, surname: str, custom_reference_id: str, job_title: str, **data):
        raise NotImplementedError

    def create_wallet(self, request_timestamp, owner_type, currency, name):
        return NotImplementedError

    def create_card(self, request_timestamp, owner_type, owner_public_id,
                    wallet_id, type, name, emboss_line4, card_label="aff"):
        return NotImplementedError

    def add_item_to_group(self, groupId: str, id: str, type="WALLET"):
        return NotImplementedError



from vc.libs.decoratos import response_builder
from .requester_base import RequesterSoldoBase
from .schemas import ResponseInfo, UserBase, Order, OrderItem, CardResponse
from .utils import request_timestamp


class User(RequesterSoldoBase):

    @response_builder(data_schema=ResponseInfo)
    def whoami(self):
        api_path = f'/business/v2/ping/whoami'
        return self.request(api_path, method='get', headers=self.default_authorize().dict())

    @response_builder(data_schema=UserBase)
    def create(self, email: str, name:str, surname: str, custom_reference_id: str, job_title: str, **data):
        # http://apidoc-demo.soldo.com/v2/zgxiaxtcyapyoijojoef.html#add-user
        api_path = f"/business/v2/employees/"
        data.update(dict(
                surname=surname,
                name=name,
                email=email,
                custom_reference_id=custom_reference_id,
                job_title=job_title
        ))
        user = UserBase(**data).dict()
        h= self.advanced_authorize(
                data, fields=("request_timestamp", "name", "surname", "mobile_access", "web_access"),
            ).dict(by_alias=True)
        return self.request(
            api_path, method='post',
            headers=h,
            json=user)

    @response_builder(data_schema=UserBase)
    def update(self, id, **data):
        # http://apidoc-demo.soldo.com/v2/zgxiaxtcyapyoijojoef.html#update-user-data
        api_path = f"/business/v2/employees/{id}"
        return self.request(
            api_path, method='put',
            headers=self.advanced_authorize(
                data, fields=("custom_reference_id", "job_title", "mobile_number", "mobile_prefix",
                             "email", "enable_mobile_credential", "enable_web_credential"),
            ).dict(by_alias=True),
            json=data)


class Wallets(RequesterSoldoBase):

    @response_builder(data_schema=Order[OrderItem])
    def create(self, owner_type, currency, name):
        # http://apidoc-demo.soldo.com/v2/zgxiaxtcyapyoijojoef.html#update-user-data
        # {
        # "id":"6a5e0887-eaf6-4f49-95cb-6b7f245d04f3",
        # "status":"PLACED","creation_time":"2021-06-29T10:42:06",
        # "last_update_time":"2021-06-29T10:42:06","is_valid":true,"total_paid_amount":0.000000,
        # "total_paid_currency":"EUR",
        # "items":[{"id":"87d95150-41f3-46a5-9501-ca1b368696ca","itemType":"WALLET","category":"WALLET"}]}
        api_path = f"/business/v2/wallets/"
        data = dict( request_timestamp=request_timestamp(), owner_type=owner_type, currency=currency, name=name)
        h = self.advanced_authorize(
                data, fields=("request_timestamp", "owner_type", "currency", "name"),
            ).dict(by_alias=True)
        return self.request(
            api_path, method='post',
            headers=h, json=data)


class Card(RequesterSoldoBase):

    @response_builder(data_schema=Order[OrderItem])
    def create(self, owner_type, owner_public_id,
                    wallet_id, type="VIRTUAL", name=None, emboss_line4=None, card_label="aff"):
        # http://apidoc-demo.soldo.com/v2/zgxiaxtcyapyoijojoef.html#update-user-data
        api_path = f"/business/v2/cards/"
        data = dict( request_timestamp=request_timestamp(), name=name, emboss_line4=emboss_line4, owner_type=owner_type, owner_public_id= owner_public_id,
                    wallet_id=wallet_id, type=type, card_label=card_label)
        h = self.advanced_authorize(
                data
                , fields=("request_timestamp", "owner_type", "owner_public_id", "wallet_id"),
            ).dict(by_alias=True)
        return self.request(
            api_path, method='post',
            headers=h, json=data)

    @response_builder(data_schema=CardResponse)
    def get(self, card_id: str, showSensitiveData: str = None):
        api_path = f"/business/v2/cards/{card_id}"
        return self.request(
            api_path, method='get',
            params={"showSensitiveData": showSensitiveData},
            headers=self.default_authorize().dict())


class Order(RequesterSoldoBase):

    @response_builder(data_schema=Order)
    def get(self, order_id: str):
        api_path = f"/business/v2/orders/{order_id}"
        return self.request(
            api_path, method='get',
            headers=self.default_authorize().dict())


class Group(RequesterSoldoBase):

    # @response_builder(data_schema=UserBase)
    def group_write(self, groupId: str, id: str, type: str):
        # http://apidoc-demo.soldo.com/v2/zgxiaxtcyapyoijojoef.html#update-user-data
        api_path = f"/business/v2/groups/{groupId}/resource"
        data = dict( id=id, type=type)
        return self.request(
            api_path, method='post',
            headers=self.advanced_authorize(data).dict(by_alias=True), json=data)


group = Group()
user = User()
wallets = Wallets()
card = Card()
order = Order()



from vc.client.base import BaseNetworkClient


# from vc.
from .requesters.client_api import user, wallets, card, group, order


from vc.settings import Settings


class Soldo(BaseNetworkClient):

    settings = Settings({
        "ACCESS_TOKEN": "fuWnizmwkTIY3NZhDrBeyuZV5CzLtctT",
    })

    def __init__(self, name, uri,
                 api_url: str,
                 client_id: str,
                 client_secret: str,
                 group_id: str,
                 token: str,
                 filepath_private_pem: str, currency="USD", user_model=None, **config):
        data = dict(name=name, currency = currency,
                                     CLIENT_ID=client_id,
                                     CLIENT_SECRET=client_secret,
                                     API_URL=api_url,
                                     TOKEN=token,
                                     PATH_RSA_PRIVATE=filepath_private_pem,
                                     GROUP_ID = group_id, **config)
        Soldo.settings.update_config(**data)
        super().__init__(uri, user_model=user_model, **config)

    def oauth_authorize(self):
        return user.oauth_authorize()

    def whoami(self):
        return user.whoami()

    def user_update(self, id, **kwargs):
        return user.update(id, **kwargs)

    def create_wallet(self,name, owner_type):
        response_data = wallets.create(owner_type, self.settings.currency, name)
        order = response_data.data
        # if order.is_valid and order.status == "PLACED":
        #     sleep(settings.TIME_SLEEP)
        #     for item in order.items:
        #         data = self.add_item_to_group(settings.GROUP_ID, item.id)
        return response_data

    def create_user(self, email: str, name: str, surname: str, custom_reference_id: str, job_title: str, **data):
        return user.create(email, name, surname, custom_reference_id, job_title, **data)

    def get_card(self, card_id: str, showSensitiveData: str = None):
        return card.get(card_id, showSensitiveData)

    def create_card(self, owner_type, owner_public_id,
                    wallet_id, name, emboss_line4=None, type="VIRTUAL", card_label=None):
        if not card_label:
            card_label = self.settings.name

        return card.create(owner_type=owner_type, owner_public_id=owner_public_id,
                    wallet_id=wallet_id, name=name,
            emboss_line4=emboss_line4,
            type=type,
            card_label=card_label)

    def add_item_to_group(self, groupId: str, id: str, type="WALLET"):
        return group.group_write(groupId, id, type)

    def get_order(self, order_id: str):
        return order.get(order_id)
